@@ -42,8 +42,8 @@
 #include "pangofc-font.h"
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
+#include "pango-engine.h"
 #include "pango-layout.h"
-#include "pango-modules.h"
 #include "pango-impl-utils.h"
 
 #include <fontconfig/fcfreetype.h>
@@ -307,18 +307,33 @@ pango_fc_font_describe_absolute (PangoFont *font)
   return desc;
 }
 
-static PangoMap *
-pango_fc_get_shaper_map (PangoLanguage *language)
+/* Wrap shaper in PangoEngineShape to pass it through old API,
+ * from times when there were modules and engines. */
+typedef PangoEngineShape      PangoFcShapeEngine;
+typedef PangoEngineShapeClass PangoFcShapeEngineClass;
+static GType pango_fc_shape_engine_get_type (void) G_GNUC_CONST;
+G_DEFINE_TYPE (PangoFcShapeEngine, pango_fc_shape_engine, PANGO_TYPE_ENGINE_SHAPE);
+static void
+_pango_fc_shape_engine_shape (PangoEngineShape    *engine G_GNUC_UNUSED,
+			      PangoFont           *font,
+			      const char          *item_text,
+			      unsigned int         item_length,
+			      const PangoAnalysis *analysis,
+			      PangoGlyphString    *glyphs,
+			      const char          *paragraph_text,
+			      unsigned int         paragraph_length)
+{
+  _pango_fc_shape (font, item_text, item_length, analysis, glyphs,
+		   paragraph_text, paragraph_length);
+}
+static void
+pango_fc_shape_engine_class_init (PangoEngineShapeClass *class)
+{
+  class->script_shape = _pango_fc_shape_engine_shape;
+}
+static void
+pango_fc_shape_engine_init (PangoEngineShape *object)
 {
-  static guint engine_type_id = 0; /* MT-safe */
-  static guint render_type_id = 0; /* MT-safe */
-
-  if (engine_type_id == 0)
-    engine_type_id = g_quark_from_static_string (PANGO_ENGINE_TYPE_SHAPE);
-  if (render_type_id == 0)
-    render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_FC);
-
-  return pango_find_map (language, engine_type_id, render_type_id);
 }
 
 static PangoEngineShape *
@@ -326,12 +341,10 @@ pango_fc_font_find_shaper (PangoFont     *font G_GNUC_UNUSED,
 			   PangoLanguage *language,
 			   guint32        ch)
 {
-  PangoMap *shaper_map = NULL;
-  PangoScript script;
-
-  shaper_map = pango_fc_get_shaper_map (language);
-  script = pango_script_for_unichar (ch);
-  return (PangoEngineShape *)pango_map_get_engine (shaper_map, script);
+  static PangoEngineShape *shaper;
+  if (g_once_init_enter (&shaper))
+    g_once_init_leave (&shaper, g_object_new (pango_fc_shape_engine_get_type(), NULL));
+  return shaper;
 }
 
 static PangoCoverage *
@@ -575,6 +588,7 @@ pango_fc_font_get_metrics (PangoFont     *font,
 	PangoRectangle extents;
 	const char *sample_str = pango_language_get_sample_string (language);
 	PangoFontDescription *desc = pango_font_describe_with_absolute_size (font);
+	gulong sample_str_width;
 
         layout = pango_layout_new (context);
 	pango_layout_set_font_description (layout, desc);
@@ -583,7 +597,9 @@ pango_fc_font_get_metrics (PangoFont     *font,
 	pango_layout_set_text (layout, sample_str, -1);
 	pango_layout_get_extents (layout, NULL, &extents);
 
-	info->metrics->approximate_char_width = extents.width / pango_utf8_strwidth (sample_str);
+	sample_str_width = pango_utf8_strwidth (sample_str);
+	g_assert (sample_str_width > 0);
+	info->metrics->approximate_char_width = extents.width / sample_str_width;
 
 	pango_layout_set_text (layout, "0123456789", -1);
 	info->metrics->approximate_digit_width = max_glyph_width (layout);
@@ -958,8 +974,10 @@ get_per_char (FT_Face      face,
  * @fcfont: a #PangoFcFont
  * @load_flags: flags to pass to FT_Load_Glyph()
  * @glyph: the glyph index to load
- * @ink_rect: location to store ink extents of the glyph, or %NULL
- * @logical_rect: location to store logical extents of the glyph or %NULL
+ * @ink_rect: (out) (optional): location to store ink extents of the
+ *   glyph, or %NULL
+ * @logical_rect: (out) (optional): location to store logical extents
+ *   of the glyph or %NULL
  *
  * Gets the extents of a single glyph from a font. The extents are in
  * user space; that is, they are not transformed by any matrix in effect
